{  simg -- command line applications for deterministic image processing
   Version 20210727
   Copyright (C) 1999-2021 Stefan Ziegenbalg
  
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 3 as
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, see http://www.gnu.org/licenses/.
!}

{@name bgdenoise}
{@description Denoising of dark regions.}

{$I directives.pas}
uses simage,getparam;
var par                 : tparams;
    cc                  : CSImg_Config;
    img,fb              : TSImg_FPImage;
    mask                : TSImg_RDImage;
    fi,fo               : ansistring;
    rad,qh,ql,fwhm      : double;
    bs,bm               : double;
    mi                  : CSImg_MetaInfo;
    i                   : longint;

begin
    with par do
    begin
        init;
        paramusage:='[<infile> [<outfile>]]';
        fi:='';      defparam('<infile>','Input image (default: stdin)',fi);
        fo:='';      defparam('<outfile>','Output image (default: stdout)',fo);
        rad:=-1;     defparam('-r','Maximum filter radius. Default: 4 times the input FWHM', rad);
        qh:=0.8;     defparam('-qh','No denoising above that quantile. Default: 80%.',qh);
        ql:=-1;      defparam('-ql','Maximum denoising below then quantile. Default: 25% of the value given by -qh.',ql);
        fwhm:=-1;    defparam('-w','Input FWHM. Default: auto-detect',fwhm);
        bs:=-1;      defparam('-s','Approximate background using that radius.',bs);
        bm:=7.5;     defparam('-m','Median filter used for background approximation. Ignored if -s is not given. Default: 7.5.',bm);
                     defparam('-o','Output image',fo); deffmt('-o','<outfile>');
                     defparam('-h','Help',helpf);
        run;
        done;
    end;
    cc:=CSImg_Config.create(SImg_Config_Default);
    cc.debug(cc.name+': fi='+fi+' fo='+fo+' rad='+real2str(rad)+' ql='+real2str(ql)+' qh='+real2str(qh)+' bs='+real2str(bs)+' bm='+real2str(bm));
    if ql<1e-3 then ql:=0.25*qh;

    mi:=CSImg_MetaInfo.create;
    cc.Info(cc.name+': Reading input image '+fi);
    SImg_ReadImageFile(cc,fi,img,mi);

    if fwhm<0.1 then
    begin
        if mi.get('FWHMPIX',fwhm) then cc.Info(cc.name+': Read PSF FWHM width from input image: ' +real2str(fwhm,-1))
        else begin
            fwhm:=SImg_GetSharpnessFWHM(cc,img);
            cc.Info(cc.name+': Auto-detected input FWHM: ' +real2str(fwhm,-1));
        end;
    end;
    if fwhm<1.5 then fwhm:=1.5;
    if rad<fwhm then rad:=4*fwhm;

    fb.pt:=nil;
    if bs>rad then
    begin
        cc.Info(cc.name+': Background approximation using ' +int2str(cc.threads) + ' threads');
        mask.pt:=nil;
        SImg_FBGApprox(cc, img,fb,mask, [bs],1,0.5, [], bm,0.6, false);
//      SImg_WriteImageFile(cc,'bgdenoise-fb.fits',fb);
        for i:=0 to fb.size-1 do
            img.pt[i]-=fb.pt[i];
    end;

    cc.Info(cc.name+': Denoising using up to ' +int2str(cc.threads) + ' threads');
    SImg_BGDenoise(cc, img, fwhm,rad,qh,ql);

    if fb.pt<>nil then
    begin
        for i:=0 to fb.size-1 do
            img.pt[i]+=fb.pt[i];
        SImg_Free(cc,fb);
    end;

    cc.Info(cc.name+': Writing output file '+fo);
    SImg_WriteImageFile(cc,fo,img,mi);

    SImg_Free(cc,img);
    cc.destroy;
end.
