{  simg -- command line applications for deterministic image processing
   Version 20210727
   Copyright (C) 1999-2021 Stefan Ziegenbalg
  
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 3 as
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, see http://www.gnu.org/licenses/.
!}

{@name cut}
{@description Cuts an image}

{$I directives.pas}
uses simage,getparam,sysutils;
var par                         : tparams;
    cc                          : CSImg_Config;
    fi,fo,val,com               : ansistring;
    x1,y1,x2,y2,x,y,c1,c2       : longint;
    xc,yc,xc2,yc2               : double;
    bx1,by1,bx2,by2,b,bx,by,rot : longint;
    bg                          : double;
    img                         : TSImg_FPImage;
    nl,fc,fm                    : boolean;
    i                           : longint;
    mi                          : CSImg_MetaInfo;

begin
    cc:=CSImg_Config.create(SImg_Config_Default);
    with par do
    begin
        init;
        paramusage:='[<infile> [<outfile>]]';
        fi:='';    defparam('<infile>','Input image (default: stdin)', fi);
        fo:='';    defparam('<outfile>','Output image (default: stdout)',fo);
                   defparam('-h','Help',helpf);
        b:=0;      defparam('-b','Border',b);
                   defparam('-bx','X border',         @bx,   @b,  tplongint,0);
                   defparam('-by','Y border',         @by,   @b,  tplongint,0);
                   defparam('-bx1','Left x border',   @bx1,  @bx, tplongint,0);
                   defparam('-bx2','Right x border',  @bx2,  @bx, tplongint,0);
                   defparam('-by1','Top y border',    @by1,  @by, tplongint,0);
                   defparam('-by2','Bottom y border', @by2,  @by, tplongint,0);
                   defparam('-x1','Left x',x1);
                   defparam('-x2','Right x',x2);
                   defparam('-y1','Top y',y1);
                   defparam('-y2','Bottom y',y2);
                   defparam('-x','X size',x);
                   defparam('-y','Y size',y);
                   defparam('-xc','X center (useful with -x)',xc);
                   defparam('-yc','Y center (useful with -y)',yc);
        fc:=false; defparam('-fc','Find center',fc);
        fm:=false; defparam('-fm','Find Maximum',fm);
        c1:=0;     defparam('-c1','First color (default: 0)',c1);
                   defparam('-c2','Last color',c2);
        rot:=0;    defparam('-r','Rotate image clockwise in 90° steps',rot);
        nl:=false; defparam('-nl','No enlarge',nl);
        bg:=0;     defparam('-bg','Background',bg);
                   defparam('-o','Destination image',fo); deffmt('-o','<outfile>');
        run;

        mi:=CSImg_MetaInfo.create;
        cc.Info('cut: Reading input image '+fi);
        SImg_ReadImageFile(cc,fi,img,mi);

        i:=0;
        if fc then i+=1;
        if fm then i+=1;
        if i>1 then
        begin
            cc.warning('-fc, -fm, -ac can''t be used together, ignored');
            fc:=false;
            fm:=false;
        end;
        if (vardef(@xc) and vardef(@yc)) then
        begin
            if fc then cc.warning('-fc is ignored');
            if fm then cc.warning('-fm is ignored');
        end;
        if fc then SImg_FindCenter(cc,img,xc2,yc2);
        if fm then SImg_FindMaximum(cc,img,xc2,yc2);

        if vardef(@x) then
        begin
            if vardef(@bx1) or vardef(@bx2) or vardef(@x1) or vardef(@x2) then cc.error('-x and -b, -bx, -bx1, -bx2, -x1, -x2 can''t be used together')
            else begin
                if not vardef(@xc) then
                begin
                    if fc or fm then xc:=xc2
                    else xc:=(img.nx-1)/2;
                end;
                x1:=round(xc-x/2);
                x2:=x1+x-1;
            end;
        end
        else begin
            if vardef(@xc) then cc.warning('-xc is ignored');
            if fc then cc.warning('-fc is ignored');
            if fm then cc.warning('-fm is ignored');
            if vardef(@bx1) then
            begin
                if vardef(@x1) then cc.error('-x1 and -b, -bx, -bx1 can''t be used together')
                else x1:=bx1;
            end
            else if not vardef(@x1) then x1:=0;
            if vardef(@bx2) then
            begin
                if vardef(@x2) then cc.error('-x2 abd -b, -bx, -bx2 can''t be used together')
                else x2:=img.nx-bx2-1;
            end
            else if not vardef(@x2) then x2:=img.nx-1;
        end;

        if vardef(@y) then
        begin
            if vardef(@by1) or vardef(@by2) or vardef(@y1) or vardef(@y2) then cc.error('-y and -b, -by, -by1, -by2, -y1, -y2 can''t be used together')
            else begin
                if not vardef(@yc) then
                begin
                    if fc or fm then yc:=yc2
                    else yc:=(img.ny-1)/2;
                end;
                y1:=round(yc-y/2);
                y2:=y1+y-1;
            end;
        end
        else begin
            if vardef(@yc) then cc.warning('-yc is ignored');
            if fc then cc.warning('-fc is ignored');
            if fm then cc.warning('-fm is ignored');
            if vardef(@by1) then
            begin
                if vardef(@y1) then cc.error('-y1 and -b, -by, -by1 can''t be used together')
                else y1:=by1;
            end
            else if not vardef(@y1) then y1:=0;
            if vardef(@by2) then
            begin
                 if vardef(@y2) then cc.error('-y2 and -b, -by, -by2 can''t be used together')
                                else y2:=img.ny-by2-1;
            end
            else if not vardef(@y2) then y2:=img.ny-1;
        end;

        if not vardef(@c2) then c2:=img.nc-1;
        if nl then
        begin
            if x1<0 then x1:=0;
            if y1<0 then y1:=0;
            if c1<0 then c1:=0;
            if x2>=img.nx then x2:=img.nx-1;
            if y2>=img.ny then y2:=img.ny-1;
            if c2>=img.nc then c2:=img.nc-1;
        end;

        done;
        end;

    cc.debug('cut: values: source='+fi+' dest='+fo+' x1='+int2str(x1)+' y1='+int2str(y1)+' x2='+int2str(x2)+' y2='+int2str(y2)+' c1='+int2str(c1)+' c2='+int2str(c2)+' bg='+real2str(bg));

    SImg_Cut(cc, img,img, x1,y1,x2,y2,c1,c2, rot, bg);

    try
        if x2<x1 then x1:=x2;
        if y2<y1 then y1:=y2;
        if mi.get('BAYERPAT',val,@com) then mi.add('BAYERPAT',''''+SImg_BayerFlipPattern(cc,(x1 and 1=1), (y1 and 1=1), trimquotes(val))+'''', com);
    except
        SImg_OutputCatchedException(cc);
    end;

    cc.Info('cut: Writing output image '+fo);
    SImg_WriteImageFile(cc,fo,img,mi);

    SImg_Free(cc,img);
    cc.destroy;
end.
