{  simg -- command line applications for deterministic image processing
   Version 20210727
   Copyright (C) 1999-2021 Stefan Ziegenbalg
  
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 3 as
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, see http://www.gnu.org/licenses/.
!}

{@name draw}
{@description Generates a specified pattern}

{$I directives.pas}
uses simage,getparam,sysutils;
var par              : tparams;
    cc               : CSImg_Config;
    img              : TSImg_FPImage;
    fo,s             : string;
    nx,ny,nc         : longint;
    gauss,mh,rn,gn   : boolean;
    sx,sy,sc,re      : double;

begin
    with par do
    begin
        init;
        paramusage:='[<outfile>]';
        fo:='';       defparam('<outfile>','Destination image (default: stdout)',fo);
                      defparam('-h','Help',helpf);
        gauss:=false; defparam('-gauss','Draw Gauss curve (default)',gauss);
        mh:=false;    defparam('-mh','Draw Mexican hat curve',mh);
        gn:=false;    defparam('-gn','Draw normal distributed noise',gn);
        rn:=false;    defparam('-rn','Draw rectangular distributed noise',rn);
        nx:=512;      defparam('-nx','X size (default: 512)',nx);
        ny:=512;      defparam('-ny','Y size (default: 512)',ny);
        nc:=1;        defparam('-nc','Number of colors (default: 1)',nc);
        sx:=64;       defparam('-sx','sigmax (Gauss and mh only, default: 64)',sx);
        sy:=64;       defparam('-sy','sigmay (Gauss and mh only, default: 64)',sy);
        sc:=1;        defparam('-sc','sigmac (Gauss and mh only, default: 1)',sc);
        re:=1;        defparam('-re','Radius exponent(Gauss, default: 1=real Gauss curve)',re);
        run;
        s:='';
        if gauss then s:='gauss';
        if mh then
        begin
            if s='' then s:='mh'
            else paramerr(format('-%s and -%s can''t be defined both',[s,'mh']));
        end;
        if gn then
        begin
            if s='' then s:='gn'
            else paramerr(format('-%s and -%s can''t be defined both',[s,'gn']));
        end;
        if rn then
        begin
            if s='' then s:='rn'
            else paramerr(format('-%s and -%s can''t be defined both',[s,'rn']));
        end;
        if s='' then
        begin
            s:='gauss';
            gauss:=true;
        end;
        done;
    end;
    cc:=CSImg_Config.create(SImg_Config_Default);
    cc.debug('draw '+s+': nx='+int2str(nx)+' ny='+int2str(ny)+' nc='+int2str(nc)+'   sx='+real2str(sx)+' sy='+real2str(sy)+' sc='+real2str(sc)+' re='+real2str(re));

    if gauss then SImg_Gauss(cc, img, nx,ny,nc, sx,sy,sc, re);
    if mh then SImg_MexHat(cc, img, nx,ny,nc, sx,sy,sc);
    if rn then SImg_Noise_Rect(cc, img, nx,ny,nc);
    if gn then SImg_Noise_Norm(cc, img, nx,ny,nc);

    SImg_WriteImageFile(cc,fo,img);
    SImg_Free(cc,img);
    writeln(stderr);
    cc.destroy;
end.
