{  simg -- command line applications for deterministic image processing
   Version 20210727
   Copyright (C) 1999-2021 Stefan Ziegenbalg
  
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 3 as
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, see http://www.gnu.org/licenses/.
!}

{@name ftest}
{@description FFT benchmark}

{$I directives.pas}
uses simage,getparam,compatun;
var par      : tparams;
    cc       : CSImg_Config;
    img      : TSImg_FPImage;
    fi,fo    : ansistring;
    it       : longint;
    pr,flops : double;
    time     : double;
    nopad    : boolean;

begin
    with par do
    begin
        init;
        paramusage:='[<infile> [<outfile>]]';
        fi:='';         defparam('<infile>','Source image (default: stdin)',fi);
        fo:='';         defparam('<outfile>','Destination image (default: stdout)',fo);
                        defparam('-h','Help',helpf);
        it:=1;          defparam('-i','Iterations',it);
        nopad:=false;   defparam('-np','Disable padding',nopad);
                        defparam('-o','Destination image',fo); deffmt('-o','<outfile>');
        run;
        done;
    end;
    cc:=CSImg_Config.create(SImg_Config_Default);
    cc.debug('ftest: values: source='+fi+' dest='+fo+' it='+real2str(it)+' padding='+boolean2str(not nopad));

    cc.info('ftest: Reading input image '+fi);
    SImg_ReadImageFile(cc,fi,img);

    cc.info('ftest: FFT benchmark using '+int2str(cc.threads)+' threads');
    SImg_FFTTest(cc,img,not nopad,it,pr,flops,time);
    cc.info(real2str(time,-3)+' s    ' + real2str(pr*1e-3,-1)+' MPixel/s    '+real2str(flops*1e-6,-2)+' GFlops/s');

    cc.info('ftest: Writing output image '+fo);
    SImg_WriteImageFile(cc,fo,img);
    SImg_Free(cc,img);
    cc.destroy;
end.
