{  simg -- command line applications for deterministic image processing
   Version 20210727
   Copyright (C) 1999-2021 Stefan Ziegenbalg
  
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 3 as
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, see http://www.gnu.org/licenses/.
!}

{@name minmax}
{@description Get minimum or maximum of two images.}

{$I directives.pas}
uses simage,getparam;
var par            : tparams;
    cc             : CSImg_Config;
    fi,fj          : TSImg_FPImage;
    fi1,fi2,fo,v,c : ansistring;
    f1,f2          : double;
    mi1,mi2        : CSImg_MetaInfo;
    mi             : boolean;

begin
    with par do
    begin
        init;
        paramusage:='<infile1> [<infile2> [<outfile>]]';
        fi1:='';    defparam('<infile1>','Input image 1',@fi1,tpastr,rapdreq);
        fi2:='';    defparam('<infile2>','Input image 2 (default: stdin)',fi2);
        fo:='';     defparam('<outfile>','Output image (default: stdout)',fo);
                    defparam('-h','Help',helpf);
        f1:=1;      defparam('-f1','Factor for image 1 (default: 1)',f1);
        f2:=1;      defparam('-f2','Factor for image 2 (default: 1)',f2);
        mi:=false;  defparam('-min','Get minimum (default: get maximum)',mi);
                    defparam('-o','Destination image',fo); deffmt('-o','<outfile>');
        run;
        done;
    end;
    cc:=CSImg_Config.create(SImg_Config_Default);
    cc.debug('minmax: src1='+fi1+' src2='+fi2+' dest='+fo+' f1='+real2str(f1)+' f2='+real2str(f2));

    mi1:=CSImg_MetaInfo.create;
    cc.Info('minmax: Reading 1st. input image '+fi1);
    SImg_ReadImageFile(cc,fi1,fi,mi1);

    cc.Info('minmax: Reading 2nd. input image '+fi2);
    SImg_ReadImageFile(cc,fi2,fj,mi1);

    SImg_minmax(cc,mi,fi,fj,fi,f1,f2);
    SImg_Free(cc,fj);

    mi2:=CSImg_MetaInfo.create;
    if mi1.get('BAYERPAT',v,@c) then mi2.add('BAYERPAT',v,c);
    cc.Info('minmax: Writing output image '+fo);
    SImg_WriteImageFile(cc,fo,fi,mi2);
    SImg_Free(cc,fi);
    cc.destroy;
end.
