{  simg -- command line applications for deterministic image processing
   Version 20210727
   Copyright (C) 1999-2021 Stefan Ziegenbalg
  
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 3 as
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, see http://www.gnu.org/licenses/.
!}

{@name mul}
{@description Multiplies two images (with variable exponent)}

{$I directives.pas}
uses simage,getparam;
var par            : tparams;
    cc             : CSImg_Config;
    fi,fj          : TSImg_FPImage;
    fi1,fi2,fo,v,c : ansistring;
    e1,e2          : double;
    mi1,mi2        : CSImg_MetaInfo;

begin
    with par do
    begin
        init;
        paramusage:='<infile1> [<infile2> [<outfile>]]';
        fi1:=''; defparam('<infile1>','Source image 1',@fi1,tpastr,rapdreq);
        fi2:=''; defparam('<infile2>','Source image 2 (default: stdin)',fi2);
        fo:='';  defparam('<outfile>','Destination image (default: stdout)',fo);
                 defparam('-h','help',helpf);
        e1:=1;   defparam('-e1','exponent of image 1 (default: 1)',e1);
        e2:=1;   defparam('-e2','exponent of image 2 (default: 1)',e2);
                 defparam('-o','Destination image',fo); deffmt('-o','<outfile>');
        run;
        done;
    end;
    cc:=CSImg_Config.create(SImg_Config_Default);
    cc.debug('mul: src1='+fi1+' src2='+fi2+' dest='+fo+' e1='+real2str(e1)+' e2='+real2str(e2));

    mi1:=CSImg_MetaInfo.create;
    cc.Info('mul: Reading 1st. input image '+fi1);
    SImg_ReadImageFile(cc,fi1,fi,mi1);

    cc.Info('mul: Reading 2nd. input image '+fi2);
    SImg_ReadImageFile(cc,fi2,fj,mi1);

    SImg_Mul(cc,fi,fj,fi,e1,e2);
    SImg_Free(cc,fj);

    mi2:=CSImg_MetaInfo.create;
    if mi1.get('BAYERPAT',v,@c) then mi2.add('BAYERPAT',v,c);
    cc.Info('mul: Writing output image '+fo);
    SImg_WriteImageFile(cc,fo,fi,mi2);
    SImg_Free(cc,fi);

    cc.destroy;
end.
