{  simg -- image processing library
   Version 20210727
   Copyright (C) 1999-2021 Stefan Ziegenbalg
  
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 3 as
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, see http://www.gnu.org/licenses/.
!}

{******* SImg_NumToSetFile ***********************************************}
function SImg_NumToSetFile(const s:ansistring;n:longint):ansistring;
var i : longint;
begin
    result:=s;
    i:=length(result);
    while (n>0) and (i>0) and (result[i]<>dirsep) do
    begin
        while ((result[i]<'0') or (result[i]>'9')) and (i>0) and (result[i]<>dirsep) do
            dec(i);
    if (i>0) and (result[i]<>dirsep) then
    begin
        n+=ord(result[i])-48;
        result[i]:=chr(48+(n mod 10));
        n:=n div 10;
        i-=1;
    end;
    end;
    if n>0 then result:='';
end;


{******* SImg_LastFileSetNum *********************************************}
function SImg_LastFileSetNum(const s:ansistring):longint;
var i,j : longint;
begin
    result:=0;
    i:=length(s); j:=1;
    while (i>0) and (s[i]<>dirsep) do
    begin
        if (s[i]>='0') and (s[i]<='9') then
        begin
            result+=j*(57-ord(s[i]));
            j*=10;
        end;
        i-=1;
    end;
end;


{*************************************************************************}
{******* CSImg_ImageSet **************************************************}
{*************************************************************************}
{******* init ************************************************************}
constructor CSImg_ImageSet.init;
begin
    getMax:=-1;
    cnx:=true;
    cnx:=true;
    cnc:=true;
    nx:=-1;
    img0.pt:=nil;
    mi0:=nil;
end;

{******* done ************************************************************}
destructor CSImg_ImageSet.done;
begin
if img0.pt<>nil then SImg_Free(SImg_Config_Default, img0);
if mi0<>nil then mi0.destroy;
end;

{******* GetImage ********************************************************}
procedure CSImg_ImageSet.getimage(cc:CSImg_Config; var img:TSImg_FPImage; mi:CSImg_MetaInfo; i:longint);
begin
    if cc=nil then cc:=SImg_Config_Default;
    cc.debug('CSImg_ImageSet.getimage: called with: img='+SImg_Infostr2(img)+' i='+int2str(i));
    if i=0 then
    begin
        if img0.pt=nil then
        begin
            if mi0<>nil then mi0.reset
            else mi0:=CSImg_MetaInfo.create;
            getimage_int(cc,img0,mi0,0);
            if not SImg_Recover(cc,img0) then cc.error('CSImg_ImageSet.getimage: invalid image');
        end;
        SImg_Duplicate(cc,img0,img,true);
        if mi<>nil then mi.add(mi0);
    end else
    begin
        if img0.pt<>nil then SImg_Free(cc,img0);
        getimage_int(cc,img,mi,i);
        if not SImg_Recover(cc,img) then cc.error('CSImg_ImageSet.getimage: invalid image');
    end;
    if (nx>0) and ((cnx and (img.nx<>nx)) or (cny and (img.ny<>ny)) or (cnc and (img.nc<>nc))) then cc.error('CSImg_ImageSet.getimage: image '+int2str(i)+': invalid dimensions');
    nx:=img.nx;
    ny:=img.ny;
    nc:=img.nc;
end;

procedure CSImg_ImageSet.getimage(cc:CSImg_Config; var img:TSImg_FPImage; i:longint);
begin
    getimage(cc,img,nil,i);
end;


{******* info ************************************************************}
function CSImg_ImageSet.info:shortstring;
begin
    info:='undefined image set';
end;

{******* getMI ***********************************************************}
// return meta information, preferably from first image
procedure CSImg_ImageSet.getMI0(cc:CSImg_Config; mi:CSImg_MetaInfo);
begin
    if mi=nil then exit;
    if mi0=nil then
    begin
        mi0:=CSImg_MetaInfo.create;
        getimage_int(cc,img0,mi0,0);
    end;
    mi.add(mi0)
end;

{*************************************************************************}
{******* CSImg_ImageSet_Files ********************************************}
{*************************************************************************}
{******* init ************************************************************}
constructor CSImg_ImageSet_Files.init(cc:CSImg_Config; const pattern:ansistring);
var i    : longint;
    s    : ansistring;
begin
    inherited init;
    if cc=nil then cc:=SImg_Config_Default;
    cc.debug('CSImg_ImageSet_Files.init: called with pattern: `'+pattern+'''');
    files:=nil; files_reserved:=0;

    getMax:=-1; i:=-1;
    repeat
        inc(i);
        s:=SImg_NumToSetFile(pattern,i);
        if (s<>'') and (FileExists(s)) then addfile(s);
    until getMax<i;

    if (getMax<0) then findfiles('',pattern);

    cc.debug('CSImg_ImageSet_Files.init: contains '+int2str(getMax+1)+' images');
    if getMax<0 then SImg_Error('CSImg_ImageSet_Files.init: can''t findfiles matching to pattern `'+pattern+'''');
end;

{******* findfile ********************************************************}
procedure CSImg_ImageSet_Files.findfiles(const path,pattern:ansistring);
var sr   : TSearchRec;
    s,r  : ansistring;
    i    : longint;
begin
    i:=pos(DirectorySeparator,pattern);
    if i>1 then
    begin
        s:=copy(pattern,1,i-1);
        r:=copy(pattern,i+1,length(pattern)-i);
        i:=faDirectory;
    end else
    begin
        s:=pattern;
        r:='';
        i:=faAnyFile;
    end;

    if findfirst(path+s,i,sr)=0 then
    repeat
        if (r<>'') and (sr.attr and faDirectory=faDirectory) then findfiles(path+sr.name+DirectorySeparator,r)
        else if (r='') and (sr.attr and faDirectory=0) then addfile(path+sr.name);
        until findnext(sr)<>0;

    findclose(sr);
end;

{******* addfile *********************************************************}
procedure CSImg_ImageSet_Files.addfile(const fn:ansistring);
begin
    inc(getMax);
    if getMax>=files_reserved then
    begin
        files_reserved+=256;
        reallocmem(files,files_reserved*sizeof(ansistring));
        fillchar(files[getMax], 256*sizeof(ansistring), 0);
    end;
    files[getMax]:=fn;
end;

{******* getimage_int ****************************************************}
procedure CSImg_ImageSet_Files.getimage_int(cc:CSImg_Config; var fi:TSImg_FPImage; mi:CSImg_MetaInfo; i:longint);
begin
    if cc=nil then cc:=SImg_Config_Default;
    if (i<0) or (i>getMax) then cc.Error('CSImg_ImagesSet_Files.getimage: image number out of range');
    cc.info('Loading file '+int2str(i+1)+'/'+int2str(getMax+1)+': '+files[i]);
    SImg_ReadImageFile(cc,files[i],fi,mi);
end;

{******* info ************************************************************}
function CSImg_ImageSet_Files.info:shortstring;
begin
    info:='Set('+files[0]+'..'+files[getMax]+')';
end;

{******* done ************************************************************}
destructor CSImg_ImageSet_Files.done;
var i   : longint;
begin
    SImg_Config_Default.debug('CSImg_ImageSet_Files.done:...');
    for i:=0 to getmax do
        files[i]:='';
    freemem(files);
    inherited done;
end;
