{  simg -- test programs
   Version 20210727
   Copyright (C) 1999-2021 Stefan Ziegenbalg
  
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 3 as
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, see http://www.gnu.org/licenses/.
!}

{$mode objfpc}
uses sysutils;

function moment(m,n:longint; mu:double):double;
var k,kz,i   : longint;
   r,s,p     : double;
begin
    kz:=round((mu+n+1)*10);
    p:=1;
    r:=0;
    for k:=-n to kz do
    begin
        s:=1;
        for i:=1 to m do
            s*=k;
        r+=s*p;
        p*=(mu+n)/(k+n+1);
    end;
    result:=r*exp(-mu-n);
end;

function momentCrop(m,n:longint; mu:double):double;
var k,kz,i  : longint;
   r,s,p    : double;
begin
    kz:=round((mu+n+1)*10);
    p:=1;
    r:=0;
    for k:=-n to kz do
    begin
        s:=1;
        for i:=1 to m do
            s*=k;
        if (m=0) or (k>0) then r+=s*p;
        p*=(mu+n)/(k+n+1);
    end;
    result:=r*exp(-mu-n);
end;

function varCrop(n:longint; mu:double):double;
begin
    result:=momentCrop(2,n,mu) - sqr(momentCrop(1,n,mu));
end;

function y0(n:longint; mu:double):double;
var k,kz        : longint;
    r1,r0,s,p   : double;
begin
    kz:=round((mu+n+1)*5);
    p:=1;
    r1:=0;
    r0:=0;
    for k:=-n to kz do
    begin
        s:=p/(1e-3+sqr(n));
        r1+=s*k;
        r0+=s;
        p*=(mu+n)/(k+n+1);
    end;
    result:=r1/r0;
end;

function y1(n:longint; mu:double):double;
var k,kz       : longint;
    r1,r0,s,p  : double;
begin
    kz:=round((mu+n+1)*5);
    p:=1;
    r1:=0;
    r0:=0;
    for k:=-n to kz do
    begin
        s:=p/(abs(k)+1+sqr(n));
        r1+=s*k;
        r0+=s;
        p*=(mu+n)/(k+n+1);
    end;
    result:=r1/r0;
end;

function y2(n:longint; mu:double):double;
var k,kz,l,lmin  : longint;
    r1,r0,s,p    : double;
begin
    kz:=round((mu+n+1)*5);
    p:=1;
    r1:=0;
    r0:=0;
    lmin:=n+4;
    for k:=-n to kz do
    begin
        l:=k-4;
        if l<lmin then l:=lmin;
        s:=p/(l+sqr(n));
        r1+=s*k;
        r0+=s;
        p*=(mu+n)/(k+n+1);
    end;
    result:=r1/r0;
end;

function y3(n:longint; mu:double):double;
var k,kz,l     : longint;
    r1,r0,s,p  : double;
begin
    kz:=round((mu+n+1)*5);
    p:=1;
    r1:=0;
    r0:=0;
    for k:=-n to kz do
    begin
        if k<1 then l:=1
        else l:=k;
        s:=p/(l+sqr(n));
        if k>0 then r1+=s*k;
        r0+=s;
        p*=(mu+n)/(k+n+1);
    end;
    result:=r1/r0;
end;

function y4(n:longint; mu,bg:double):double;
var k,kz         : longint;
    r1,r0,s,p,u,a: double;
begin
    kz:=round((mu+n+1)*5);
    p:=1;
    r1:=0;
    r0:=0;
    a:=sqr(bg);
    if a<5 then a:=5;
    for k:=-n to kz do
    begin
        if k<a then u:=0
        else if k<2*a then u:=sqr(k-a)/(2*a)
        else u:=k-1.5*a;
        s:=p/(1+sqr(bg)+u);
        r1+=s*k;
        r0+=s;
        p*=(mu+n)/(k+n+1);
    end;
    result:=r1/r0;
end;

function y5(n:longint; mu,bg:double):double;
var k,kz         : longint;
    r1,r0,s,p,u,a,b: double;
begin
    if bg<1e-30 then bg:=1e-30
    else if bg>1 then bg:=sqr(bg);
    kz:=round((mu+n+1)*5);
    p:=1;
    r1:=0;
    r0:=0;
    a:=bg;
    b:=25/bg;
    for k:=-n to kz do
    begin
        if k<a then u:=0
        else if k<a+b then u:=sqr(k-a)/(2*b)
        else u:=k-a-0.5*b;
        s:=p/(1e-2+bg+u);
        r1+=s*k;
        r0+=s;
        p*=(mu+n)/(k+n+1);
    end;
    result:=r1/r0;
end;


var n,m,i       : longint;
    mu,y,mup,yp : double;

begin
    writeln('Testing moments');
    for n:=0 to 3 do
        writeln('n=',n,':    m_0(µ=1)=',moment(0,n,1):0:6,'    m_0(µ=2)=',moment(0,n,2):0:6,'    m_0(µ=3)=',moment(0,n,3):0:6);
    for n:=0 to 3 do
        writeln('n=',n,':    µ=m_1(µ=1)=',moment(1,n,1):0:6,'    µ=m_1(µ=2)=',moment(1,n,2):0:6, '    µ=m_1(µ=3)=',moment(1,n,3):0:6);

    mu:=0;
    writeln(#10'y0(µ) (no cropping, photon noise estimator 1e-3');
    for n:=0 to 12 do
    begin
        write('n^2=',sqr(n):3,':');
        for i:=0 to 7 do
        begin
            case i of
                0  : mu:=0.1;
                1  : mu:=0.5;
                2  : mu:=1;
                3  : mu:=2;
                4  : mu:=4;
                5  : mu:=8;
                6  : mu:=16;
                7  : mu:=32;
            end;
            write('    y(µ=',mu:0:1,')=', y0(n,mu):0:6);
        end;
        writeln;
    end;

    writeln(#10'y1(µ) (no cropping, photon noise estimator |k|+1 )');
    for n:=0 to 10 do
    begin
        write('n^2=',sqr(n):3,':');
        for i:=0 to 4 do
        begin
            case i of
                0  : mu:=0.1;
                1  : mu:=0.5;
                2  : mu:=1;
                3  : mu:=2;
                4  : mu:=4;
            end;
            write('    y(µ=',mu:0:1,')=', y1(n,mu):0:6);
        end;
        writeln;
    end;

    writeln(#10'y2(µ) (no cropping, photon noise estimator max(sqrt(n)+4,k-4))');
    for n:=0 to 12 do
    begin
        write('n^2=',sqr(n):3,':');
        for i:=0 to 7 do
        begin
            case i of
                0  : mu:=0.1;
                1  : mu:=0.5;
                2  : mu:=1;
                3  : mu:=2;
                4  : mu:=4;
                5  : mu:=8;
                6  : mu:=16;
                7  : mu:=32;
            end;
            write('    y(µ=',mu:0:1,')=', y2(n,mu):0:6);
        end;
        writeln;
    end;

    mup:=0; yp:=0;
    writeln(#10'y3(µ) (cropping, photon noise estimator max(k,1))');
    for n:=0 to 10 do
    begin
        write('n^2=',sqr(n):3,':');
        write('  Var[µ=0]=',varCrop(sqr(n),0):0:4);

//        write('  Var[µ=1]=',varCrop(sqr(n),1):0:4);
        for i:=-1 to 4 do
        begin
            case i of
                0  : mu:=0.1;
                1  : mu:=0.5;
                2  : mu:=1;
                3  : mu:=2;
                4  : mu:=4;
                else mu:=0;
            end;
            y:=y3(n,mu);
            write('    y(µ=',mu:0:1,')=', y:0:6);
            if i>=0 then write('  dµ/dy=',(mu-mup)/(y-yp):0:3,'  ');
//            if (i=0) and (n>0) then write('  Var_crop/Var_nocrop*sqr(dµ/dy)=',varCrop(sqr(n),0)/sqr(n)*sqr((mu-mup)/(y-yp)):0:3,'  ');
            yp:=y;
            mup:=mu;
        end;
        writeln;
    end;
{ result: cropping does not reduce noise at µ=0 because sqr(dµ/du)*VarCrop/Var > 1 }

(*   for m:=1 to 3 do
    begin
        writeln(#10'y4(µ) (no cropping, noise estimator 1+bg+(k<0 ? 0 : k<2*a ? sqr(k-a)/a : k-1.5*a); a=max(bg,4); bg=n*',m);
        for n:=0 to 12 do
        begin
            write('n^2=',sqr(n):3,':');
            for i:=0 to 7 do
            begin
                case i of
                    0  : mu:=0.1;
                    1  : mu:=0.5;
                    2  : mu:=1;
                    3  : mu:=2;
                    4  : mu:=4;
                    5  : mu:=8;
                    6  : mu:=16;
                    7  : mu:=32;
                end;
                write('    y(µ=',mu:0:1,')=', y4(n,mu,n*m):0:6);
            end;
            writeln;
        end;
    end; *)

    for m:=1 to 3 do
    begin
        writeln(#10'y5(µ) (no cropping, noise estimator 1e-2+bg+(k<a ? 0 : k<a+b ? sqr(k-a)/(2*b) : k-a-0.5*b); a=max(bg,4); b=4*a/bg; bg=0.25+n*',m);
        for n:=0 to 12 do
        begin
            write('n^2=',sqr(n):3,':');
            for i:=0 to 7 do
            begin
                case i of
                    0  : mu:=0.1;
                    1  : mu:=0.5;
                    2  : mu:=1;
                    3  : mu:=2;
                    4  : mu:=4;
                    5  : mu:=8;
                    6  : mu:=16;
                    7  : mu:=32;
                end;
                write('    y(µ=',mu:0:1,')=', y5(n,mu,0.25+n*m):0:6);
            end;
            writeln;
        end;
    end;

end.

