{  simg -- visualization library
   Version 20210727
   Copyright (C) 1999-2021 Stefan Ziegenbalg
  
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 3 as
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, see http://www.gnu.org/licenses/.
!}

{ ************************************************************************* }
{ ******** imagewidget events ********************************************* }
{ ************************************************************************* }
{ ******** siv_iw_motion_event ******************************************** }
function siv_iw_motion_event(widget:PGtkWidget; event:PGdkEventMotion; self:CSIV_ImageWidget):longint; cdecl;
var x,y       : longint;
    state     : TGdkModifierType;
begin
    result:=0;
    if event=nil then
    begin
        siv_warning('siv_iw_motion_event: event=nil');
        exit;
    end;
    if self=nil then
    begin
        siv_warning('siv_iw_motion_event: self=nil');
        exit;
    end;
    with self do
    begin
        if event^.is_hint>0 then gdk_window_get_pointer(event^.window, @x, @y, @state)
        else begin
            x:=round(event^.x);
            y:=round(event^.y);
            state:=event^.state;
        end;
        if (state and GDK_BUTTON3_MASK>0) and (b3_pressed) then
        begin
            b3_x-=x;
            b3_y-=y;
            with hadj^ do
            begin
                value+=b3_x;
                if value<lower then value:=lower;
                if value>upper-page_size then value:=upper-page_size;
            end;
            with vadj^ do
            begin
                value+=b3_y;
                if value<lower then value:=lower;
                if value>upper-page_size then value:=upper-page_size;
            end;
            gtk_adjustment_value_changed(hadj);
            gtk_adjustment_value_changed(vadj);
            gdk_window_get_pointer(event^.window, @x, @y, @state);
            b3_x:=x;
            b3_y:=y;
        end;
    end;
end;

{ ******** siv_iw_button_press_event ************************************** }
function siv_iw_button_press_event(widget:PGtkWidget;event:PGdkEventButton;self:CSiv_ImageWidget):longint; cdecl;
begin
    result:=0;
    if event=nil then
    begin
        siv_warning('siv_iw_press_event: event=nil');
        exit;
    end;
    if self=nil then
    begin
        siv_warning('siv_iw_press_event: self=nil');
        exit;
    end;
    with self do
    begin
        if event^.button=3 then
        begin
            b3_x:=round(event^.x);
            b3_y:=round(event^.y);
            b3_pressed:=true;
        end;
    end;
end;

{ ******** siv_iw_button_release_event ************************************ }
function siv_iw_button_release_event(widget:PGtkWidget; event:PGdkEventButton; self:CSiv_ImageWidget):longint; cdecl;
begin
    result:=0;
    if event=nil then
    begin
        siv_warning('siv_iw_release_event: event=nil');
        exit;
    end;
    if self=nil then
    begin
        siv_warning('siv_iw_release_event: self=nil');
        exit;
    end;
    with self do
    begin
        if event^.button=3 then b3_pressed:=false;
    end;
end;

{ ******** siv_iw_leave_event ********************************************* }
function siv_iw_leave_event(widget:PGtkWidget; event:PGdkEvent; self:CSiv_ImageWidget):longint; cdecl;
begin
    result:=0;
    if self=nil then
    begin
        siv_warning('siv_iw_leave_event: self=nil');
        exit;
    end;
    with self do
    begin
        b3_pressed:=false;
    end;
    result:=1;
end;

{ ******** siv_iw_size_request ******************************************** }
procedure siv_iw_size_request(widget:PGtkWidget; requisition: PGtkRequisition; self:CSIV_ImageWidget); cdecl;
begin
    if widget=nil then
    begin
        siv_warning('siv_iw_size_request: widget=nil');
        exit;
    end;
    if requisition=nil then
    begin
        siv_warning('siv_iw_size_request: requisition=nil');
        exit;
    end;
    if self=nil then
    begin
        siv_warning('siv_iw_size_request: self=nil');
        exit;
    end;
    with self do
    begin
        if fit_request then
        begin
            requisition^.width:=image.scaledWidth;
            if requisition^.width<sb_width*3 then requisition^.width:=sb_width*3;
            requisition^.width+=PSivDraw(draw)^.extra_width*2;

            requisition^.height:=image.scaledHeight;
            if requisition^.height<sb_height*3 then requisition^.height:=sb_height*3;
            requisition^.height+=PSivDraw(draw)^.extra_width*2;
        end;
        fit_request:=false;
//        writeln('iw.size_request: ',requisition^.width,'  ',requisition^.height);
    end;
end;

{ ******** siv_iw_size_allocate ******************************************* }
function siv_iw_size_allocate(widget:PGtkWidget; allocation: PGdkRectangle; self:CSIV_ImageWidget):boolean32; cdecl;
var mw,mh         : longint;
    width,height  : longint;
    vv,hv         : boolean;
    r             : double;
begin
    result:=false;
    if widget=nil then
    begin
        siv_warning('siv_iw_size_allocate: widget=nil');
        exit;
    end;
    if allocation=nil then
    begin
        siv_warning('siv_iw_size_allocate: allocation=nil');
        exit;
    end;
    if self=nil then
    begin
        siv_warning('siv_iw_size_allocate: self=nil');
        exit;
    end;
    with self do
    begin
        hv:=false;
        vv:=false;
        mw:=image.scaledWidth;
        mh:=image.scaledHeight;
        width:=allocation^.width - 2*PSivDraw(draw)^.extra_width;
        height:=allocation^.height - 2*PSivDraw(draw)^.extra_height;
        if width<mw then
        begin
            height-=sb_height;
            hv:=true;
        end;
        if height<mh then
        begin
            width-=sb_width;
            vv:=true;
            if (not hv) and (width<mw) then
            begin
                height-=sb_height;
                hv:=true;
            end;
        end;
        siv_widget_set_visible(hsb,hv);
        siv_widget_set_visible(vsb,vv);

        if width>mw then width:=mw;
        if height>mh then height:=mh;

        r:=siv_adjustment_get_pos(hadj);
        hadj^.page_size:=width;
        hadj^.upper:=mw-1;
        hadj^.page_increment:=(mw-width)*0.1;
        hadj^.value:=r*(mw-width);
        gtk_adjustment_changed(hadj);
        gtk_adjustment_value_changed(hadj);

        r:=siv_adjustment_get_pos(vadj);
        vadj^.page_size:=height;
        vadj^.upper:=mh-1;
        vadj^.page_increment:=(mh-height)*0.1;
        vadj^.value:=r*(mh-height);
        gtk_adjustment_changed(vadj);
        gtk_adjustment_value_changed(vadj);

    end;
//writeln('iw.size_allocate: ',allocation^.x,'  ',allocation^.y,'  ',allocation^.width,'  ',allocation^.height);
end;


{ ************************************************************************* }
{ ******** CSIV_ImageWidget *********************************************** }
{ ************************************************************************* }
{ ******** init *********************************************************** }
constructor CSIV_ImageWidget.init(parent:PGtkWidget; _image:CSIV_Image);
var bg_gc        : PGdkGC;
    requisition  : TGtkRequisition;
begin
    image:=_image;
    b3_pressed:=false;
    fit_request:=false;

    hadj:=PGtkAdjustment(gtk_adjustment_new(5, 0, 19, 1, 2, 10));
    vadj:=PGtkAdjustment(gtk_adjustment_new(5, 0, 19, 1, 2, 10));

    bg_gc:=gdk_gc_new(parent^.window);
    gdk_rgb_gc_set_foreground(bg_gc,$404040);
    gdk_rgb_gc_set_background(bg_gc,$000000);
    gdk_pixmap_unref(gdk_pixmap_create_from_xpm_d(parent^.window, mask, nil, @xpm_pattern_1_16));
    gdk_gc_set_fill(bg_gc,GDK_OPAQUE_STIPPLED);
    gdk_gc_set_stipple(bg_gc,mask);
    draw:=siv_draw_new(image, bg_gc, GTK_SHADOW_IN, 0);
    gtk_widget_show(draw);
    g_signal_connect(GTK_OBJECT(draw),'button_press_event',TGtkSignalFunc(@siv_iw_button_press_event),self);
    g_signal_connect(GTK_OBJECT(draw),'button_release_event',TGtkSignalFunc(@siv_iw_button_release_event),self);
    g_signal_connect(GTK_OBJECT(draw),'motion_notify_event',TGtkSignalFunc(@siv_iw_motion_event),self);
    g_signal_connect(GTK_OBJECT(draw),'leave_notify_event',TGtkSignalFunc(@siv_iw_leave_event),self);
    gtk_widget_set_events(draw,GDK_EXPOSURE_MASK
        or GDK_LEAVE_NOTIFY_MASK
        or GDK_BUTTON_PRESS_MASK
        or GDK_KEY_PRESS_MASK
        or GDK_BUTTON_RELEASE_MASK
        or GDK_BUTTON_MOTION_MASK
        or GDK_POINTER_MOTION_HINT_MASK);
    gdk_gc_unref(bg_gc);
    siv_draw_set_hadj(PSivDraw(draw),hadj);
    siv_draw_set_vadj(PSivDraw(draw),vadj);

    widget:=gtk_table_new(2,2,false);
    gtk_table_attach(GTK_TABLE(widget),draw, 0,1, 0,1, GTK_FILL or GTK_EXPAND or GTK_SHRINK, GTK_FILL or GTK_EXPAND or GTK_SHRINK, 0,0);

    vsb:=gtk_vscrollbar_new(vadj);
    gtk_widget_show(vsb);
    gtk_table_attach(GTK_TABLE(widget),vsb, 1,2, 0,1, GTK_SHRINK, GTK_FILL or GTK_EXPAND or GTK_SHRINK, 0,0);
    gtk_widget_size_request(vsb,@requisition);
    sb_width:=requisition.width;

    hsb:=gtk_hscrollbar_new(hadj);
    gtk_widget_show(hsb);
    gtk_table_attach(GTK_TABLE(widget),hsb, 0,1, 1,2, GTK_FILL or GTK_EXPAND or GTK_SHRINK, GTK_SHRINK, 0,0);
    gtk_widget_size_request(hsb,@requisition);
    sb_height:=requisition.height;

    g_signal_connect(GTK_OBJECT(widget),'size_request',TGtkSignalFunc(@siv_iw_size_request),self);
    g_signal_connect(GTK_OBJECT(widget),'size_allocate',TGtkSignalFunc(@siv_iw_size_allocate),self);
    siv_debug_end;
end;

{ ******** done *********************************************************** }
destructor CSIV_ImageWidget.done;
begin
    gdk_bitmap_unref(mask);
    siv_debug_end;
end;

{ ******** redraw ********************************************************** }
procedure CSIV_ImageWidget.redraw;
begin
    siv_draw_reset(PSivDraw(draw));
    gtk_widget_queue_resize(widget);
end;

{ ******** enable_fit_request ********************************************* }
procedure CSIV_ImageWidget.enable_fit_request;
begin
    fit_request:=true;
end;

{ ******** show *********************************************************** }
procedure CSIV_ImageWidget.show;
begin
    gtk_widget_show(widget);
end;

{ ******** hide *********************************************************** }
procedure CSIV_ImageWidget.hide;
begin
    gtk_widget_hide(widget);
end;
